﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Reflection;
using System.Windows;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;

namespace XamTreeView
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        ObservableCollection<Child> mySource = new ObservableCollection<Child>();

        public Window1()
        {
            InitializeComponent();
            LoadMySource(mySource);
            xamDataGrid1.DataSource = mySource;
        }

        void LoadMySource(ObservableCollection<Child> source)
        {
            source.Clear();
            Toy alpha = new Toy() { ID = "1", Name = "Teddy Bear" };
            Toy beta = new Toy() { ID = "2", Name = "Toy Gun" };
            Child kid = new Child() { Name = "John", ParentName = "Bob", IsExpanded = false };
            kid.Toys = new ObservableCollection<Toy>();
            kid.Toys.Add(alpha);
            kid.Toys.Add(beta);
            source.Add(kid);
        }

        private void xamDataGrid1_RecordExpanded(object sender, RecordExpandedEventArgs e)
        {
            Type recordType = ((DataRecord)e.Record).DataItem.GetType();
            if (recordType == typeof(Child))
            {
                Console.WriteLine("Record expanded.  Hide values now...");
                DataRecord dataRecord = (DataRecord)e.Record;
                ((Child)(dataRecord.DataItem)).IsExpanded = true;

                Console.WriteLine("Done");
            }
        }

        private void xamDataGrid1_RecordCollapsed(object sender, RecordCollapsedEventArgs e)
        {
            Type recordType = ((DataRecord)e.Record).DataItem.GetType();
            if (recordType == typeof(Child))
            {
                Console.WriteLine("Record collapsed.  Display values now...");
                DataRecord dataRecord = (DataRecord)e.Record;
                ((Child)(dataRecord.DataItem)).IsExpanded = false;

                Console.WriteLine("Done");
            }
        }
    }

    public class Toy
    {
        public string ID { get; set; }
        public string Name { get; set; }
    }

    public class Child : INotifyPropertyChanged
    {
        public string Name { get; set; }
        public string ParentName { get; set; }
        public ObservableCollection<Toy> Toys { get; set; }

        private bool _isExpanded;
        public bool IsExpanded
        {
            get { return _isExpanded; }
            set { SetProperty("IsExpanded", value); }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        protected void SetProperty(string propertyName, object value)
        {
            string firstChar = propertyName.Substring(0, 1).ToLower();
            string fieldName = "_" + firstChar + propertyName.Remove(0, 1);
            Type myType = GetType();
            FieldInfo myField = myType.GetField(fieldName, BindingFlags.NonPublic | BindingFlags.Instance);
            if (myField == null)
            {
                throw new Exception("Could not resolve property: " + propertyName);
            }
            object currentValue = myField.GetValue(this);
            if (currentValue != value)
            {
                myField.SetValue(this, value);
                OnPropertyChanged(propertyName);
            }
        }

        protected void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}